#include <SoftwareSerial.h>
#include <UTFT.h>
#include <UTouch.h>
#include <tinyFAT.h>
#include <strings.h>

UTFT        myGLCD(GEEE32,38,39,40,41);
UTouch      myTouch(6,5,4,3,2);

extern uint8_t SmallFont[];
extern uint8_t BigFont[];
SoftwareSerial serialCalc(10, 12);

int error;
int pos = 0;

char filename[] = "htmlcalc.htm";

void setup(void)
{
  // Initialize serial communication at 115200 baud
  Serial.begin(9600);
  serialCalc.begin(9600);

  Serial.println();

  // Setup the LCD
  myGLCD.InitLCD();
  myGLCD.setFont(SmallFont);
  myGLCD.clrScr();

  // Initialize tinyFAT
  printMsg("Initialisation...");
  error = file.initFAT();
  if (error != NO_ERROR)
  {
    printMsg("***** ERROR *****");
    printMsg(verboseError(error));
    printMsg("Process aborted.");
    printMsg("Please check the SD card before trying");
    printMsg("again.");
    while (true) {
    }
  }
  printMsg("Done !");
  printMsg("");
}

void loop(void)
{
  char bufferCmd[50];
  char c = 0;
  int i = 0;
  int command;

  while(1)
  {
    i = 0;
    clrSerial();
    clrBuffer(bufferCmd, 50);
    printMsg("Waiting for a command.");

    while(!serialCalc.available()) {
    }; // waiting for a cmd
    delay(10);
    while(1)
    {
      while(!serialCalc.available()) {
      };
      c = serialCalc.read();
      if(c == '\n') break;
      bufferCmd[i] = c;
      i++;
    }

    printMsg(bufferCmd);

    command = getCmd(bufferCmd[0]); // analysing data

    switch(command)
    {
    case 0: 
      printMsg("Please write a valid command (H : help)");
      printMsg("");
      break;
    case 1: 
      printHelp();
      break;
    case 2: 
      listFile();
      break;
    case 4: 
      sendFile(filename);
      break;
    case 6: 
      pos = 0;
      myGLCD.clrScr();
    }
  }
}

void clrSerial()
{
  while(serialCalc.available()) serialCalc.read();
}

void clrBuffer(char *buf, int size)
{
  for(int i=0; i<size; i++) buf[i] = '\0';
}

char *verboseError(byte err)
{
  switch (err)
  {
  case ERROR_MBR_READ_ERROR:
    return "Error reading MBR";
    break;
  case ERROR_MBR_SIGNATURE:
    return "MBR Signature error";
    break;
  case ERROR_MBR_INVALID_FS:
    return "Unsupported filesystem";
    break;
  case ERROR_BOOTSEC_READ_ERROR:
    return "Error reading Boot Sector";
    break;
  case ERROR_BOOTSEC_SIGNATURE:
    return "Boot Sector Signature error";
    break;
  default:
    return "Unknown error";
    break;
  }
}

void printMsg(const char* msg)
{
  if((pos+1)*13 > 240)
  {
    pos = 0;
    myGLCD.clrScr();
  }

  myGLCD.setColor(255,255,255);
  myGLCD.print(msg, LEFT, pos*13+1);
  pos++;
}

int getCmd(char cmd)
{
  if(cmd == 'H') return 1;
  if(cmd == 'L') return 2;
  if(cmd == 'O') return 3;
  if(cmd == 'S') return 4;
  if(cmd == 'E') return 5;
  if(cmd == 'C') return 6;
  return 0;
}

void printHelp(void)
{
  printMsg("***** HELP *****");
  printMsg("H            : Print help");
  printMsg("L            : create listfile");
  printMsg("O <filename> : open a file");
  printMsg("S <filename> : send a file to calc");
  printMsg("E <filename> : check if a file exists");
  printMsg("C            : clear screen");
  printMsg("****************");
  printMsg("");
}  

void listFile(void)
{
  char bufferFile[150];
  char fileSize[20];

  printMsg("Listing files...");

  error = file.findFirstFile(&file.DE);
  if (error == NO_ERROR)
  {
    printMsg("");
    clrBuffer(bufferFile, 150);

    sprintf(fileSize, "%d", file.DE.fileSize);
    strcat(bufferFile, file.DE.filename);
    strcat(bufferFile, ".");
    strcat(bufferFile, file.DE.fileext);
    strcat(bufferFile, "  Size: ");
    strcat(bufferFile, fileSize);
    printMsg(bufferFile);
  }
  else
  {
    printMsg("No files found...");
    printMsg("");
    return;
  }
  while (error == NO_ERROR)
  {
    error = file.findNextFile(&file.DE);
    if(error != NO_ERROR)
    {
      printMsg("");
      return;
    }

    clrBuffer(bufferFile, 150);

    sprintf(fileSize, "%d", file.DE.fileSize);
    strcat(bufferFile, file.DE.filename);
    strcat(bufferFile, ".");
    strcat(bufferFile, file.DE.fileext);
    strcat(bufferFile, "  Size: ");
    strcat(bufferFile, fileSize);
    printMsg(bufferFile);

    if((pos+3)*13 > 240)
    {
      printMsg("Press a key to continue...");
      clrSerial();
      delay(10);
      while(!serialCalc.available()) {
      }
    }
  }
}

/*void open_file(char *fName)
 {
 int result;
 char textBuffer;
 
 error = file.openFile(fName, FILEMODE_TEXT_READ);
 if(error == NO_ERROR)
 {
 printMsg("***** BEGIN *****");
 result = 0;
 while((result != EOF) and (result != FILE_IS_EMPTY))
 {
 result = file.readLn(&textBuffer, 1);
 
 if(result != FILE_IS_EMPTY)
 {
 serialCalc.write(textBuffer);
 }
 else
 {
 break;
 }
 }
 file.closeFile();
 }
 else
 {
 switch(error)
 {
 case ERROR_ANOTHER_FILE_OPEN:
 printMsg("* ERROR: Another file is already open.");
 break;
 case ERROR_FILE_NOT_FOUND:
 printMsg("* ERROR: File not found.");
 break;
 }
 }
 printMsg("");
 }*/

void sendFile(char *fName)
{
  int i;
  int result;
  char textBuffer[81];

  error = file.openFile(fName, FILEMODE_TEXT_READ);
  if(error == NO_ERROR)
  {
    printMsg("Sending...");
    serialCalc.write("\n\n");
    serialCalc.write(fName);
    serialCalc.write("\n**********\n");

    result=0;

    while ((result!=EOF) and (result!=FILE_IS_EMPTY))
    {
      result = 0;
      while ((result!=EOF) and (result!=FILE_IS_EMPTY))
      {
        result = file.readLn(textBuffer, 80);
        if (result != FILE_IS_EMPTY)
        {
          for(i=0; i<81; i++) if(textBuffer[i] == '\t') textBuffer[i] = ' ';
          
          if (result == BUFFER_OVERFLOW)
            serialCalc.print(textBuffer);
          else
            serialCalc.println(textBuffer);
        }
      }
      else
        printMsg("* ERROR: File is empty...");
    }

    serialCalc.write("\n**********\n");

    printMsg("Done !");
    printMsg("");
    file.closeFile();
  }
  else
  {
    switch(error)
    {
    case ERROR_ANOTHER_FILE_OPEN:
      printMsg("* ERROR: Another file is already open.");
      break;
    case ERROR_FILE_NOT_FOUND:
      printMsg("* ERROR: File not found.");
      break;
    }
  }
  printMsg("");
}

char uCase(char c)
{
  if ((c>='a') && (c<='z'))
    return (c-0x20);
  else
    return c;
}


